# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from ..limit_response import LimitResponse

__all__ = ["PropertyUpdateResponse"]

PropertyUpdateResponse: TypeAlias = List[LimitResponse]
